@@ -7,19 +7,19 @@ test_that("Importing biom files yield SummarizedExperiment objects", {
     expect_s4_class(me, "SummarizedExperiment")
     # load from object
     x1 <- biomformat::read_biom(rich_dense_file)
-    me2 <- makeTreeSEFromBiom(x1)
+    me2 <- convertFromBIOM(x1)
     expect_s4_class(me2, "SummarizedExperiment")
     expect_equal(dim(me), dim(me2))
     expect_equal(rowData(me), rowData(me2))
-    
+
     biom_object <- biomformat::read_biom(
-        system.file("extdata/testdata/Aggregated_humanization2.biom",
+        system.file("extdata", "Aggregated_humanization2.biom",
                     package="mia")
     )
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes = FALSE,
-                                rankFromPrefix = FALSE,
-                                remove.artifacts = TRUE,
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm = FALSE,
+                                rank.from.prefix = FALSE,
+                                artifact.rm = TRUE,
                                 pattern = "\"")
     # Testing no prefixes removed
     expect_true(rowData(tse) %>%
@@ -29,61 +29,61 @@ test_that("Importing biom files yield SummarizedExperiment objects", {
     expect_false(
         sapply(tolower(colnames(rowData(tse))),
                 function(x) x %in% TAXONOMY_RANKS) %>% all())
-    # Testing the remove.artifacts, since the original artifact in the biom file 
+    # Testing the artifact.rm, since the original artifact in the biom file
     # is '\"'
     expect_false(apply(rowData(tse), 2, grepl, pattern="^\"") %>% all())
-    
+
     # Testing prefixes removed
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=TRUE,
-                                rankFromPrefix=TRUE,
-                                remove.artifacts = TRUE,
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=TRUE,
+                                rank.from.prefix=TRUE,
+                                artifact.rm = TRUE,
                                 pattern = "\"")
     expect_false(rowData(tse) %>%
                     apply(2,grepl,pattern="sk__|([dkpcofgs]+)__") %>%
                     all())
-    
+
     # Testing parsing taxonomy ranks from prefixes
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=FALSE,
-                                rankFromPrefix=TRUE,
-                                remove.artifacts = TRUE,
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=FALSE,
+                                rank.from.prefix=TRUE,
+                                artifact.rm = TRUE,
                                 pattern = "\"")
     expect_true(
         sapply(tolower(colnames(rowData(tse))),
                 function(x) x %in% TAXONOMY_RANKS) %>% all())
-    
-    # Testing the remove.artifacts, the original artifact in the biom file 
+
+    # Testing the artifact.rm, the original artifact in the biom file
     # is '\"', as a test we rather try remove a non existing pattern.
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=FALSE,
-                                rankFromPrefix=FALSE,
-                                remove.artifacts = TRUE,
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=FALSE,
+                                rank.from.prefix=FALSE,
+                                artifact.rm = TRUE,
                                 pattern = "\\*|\\?")
     # with wrong pattern artifact not cleaned
     expect_true(apply(rowData(tse), 2, grepl, pattern="\"") %>% any())
-    # Testing the remove.artifacts, with the value 'auto' to automatically 
+    # Testing the artifact.rm, with the value 'auto' to automatically
     # detect the artifact and remove it (in our case the artifact is '\"').
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=FALSE,
-                                rankFromPrefix=FALSE,
-                                remove.artifacts = TRUE)
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=FALSE,
+                                rank.from.prefix=FALSE,
+                                artifact.rm = TRUE)
     # Checking if 'auto' has detected and cleaned the artifact
     expect_false(apply(rowData(tse), 2, grepl, pattern="\"") %>% any())
-    # Testing the remove.artifacts, with the value NULL to not detect or clean 
+    # Testing the artifact.rm, with the value NULL to not detect or clean
     # anything.
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=FALSE,
-                                rankFromPrefix=FALSE,
-                                remove.artifacts = FALSE)
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=FALSE,
+                                rank.from.prefix=FALSE,
+                                artifact.rm = FALSE)
     # Checking if the '\"' artifact still exists.
     expect_true(apply(rowData(tse), 2, grepl, pattern="\"") %>% any())
-    
+
     # General final test
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=TRUE,
-                                rankFromPrefix=TRUE,
-                                remove.artifacts = TRUE)
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=TRUE,
+                                rank.from.prefix=TRUE,
+                                artifact.rm = TRUE)
     # check if '\"' cleaned
     expect_false(apply(rowData(tse), 2, grepl, pattern="\"") %>% any())
     # check if taxa prefixes removed
@@ -94,16 +94,16 @@ test_that("Importing biom files yield SummarizedExperiment objects", {
     expect_true(
         sapply(tolower(colnames(rowData(tse))),
                 function(x) x %in% TAXONOMY_RANKS) %>% all())
-    
+
     # General final test with another biom file
     biom_object <- biomformat::read_biom(
         system.file("extdata", "rich_dense_otu_table.biom",
                     package = "biomformat")
     )
-    tse <- makeTreeSEFromBiom(biom_object,
-                                removeTaxaPrefixes=TRUE,
-                                rankFromPrefix=TRUE,
-                                remove.artifacts = TRUE)
+    tse <- convertFromBIOM(biom_object,
+                                prefix.rm=TRUE,
+                                rank.from.prefix=TRUE,
+                                artifact.rm = TRUE)
     # check if taxa prefixes removed
     expect_false(rowData(tse) %>%
                     apply(2,grepl,pattern="sk__|([dkpcofgs]+)__") %>%
@@ -112,25 +112,70 @@ test_that("Importing biom files yield SummarizedExperiment objects", {
     expect_true(
         sapply(tolower(colnames(rowData(tse))),
                 function(x) x %in% TAXONOMY_RANKS) %>% all())
+
+    # Check that convertToBIOM works
+    # Get errors if input is incorrect
+    expect_error( convertToBIOM() )
+    expect_error( convertToBIOM(c(1, 2, 3)) )
+    expect_error( convertToBIOM(tse, assay.type = "test") )
+    # Check that the output is correct
+    biom <- convertToBIOM(tse)
+    # Test type
+    expect_equal(biomformat::matrix_element_type(biom), "int")
+    # Assay
+    test <- as.matrix( biomformat::biom_data(biom) )
+    ref <- assay(tse)
+    expect_equal(test, ref)
+    # rowData
+    test <- as.data.frame( biomformat::observation_metadata(biom) )
+    ref <- as.data.frame( rowData(tse) )
+    expect_equal(test, ref)
+    # colData
+    test <- as.data.frame( biomformat::sample_metadata(biom) )
+    ref <- as.data.frame( colData(tse) )
+    expect_equal(test, ref)
+    # Test with empty rowData and colData and relative abundance (float) assay
+    rowData(tse) <- NULL
+    colData(tse) <- NULL
+    tse <- transformAssay(tse, method = "relabundance")
+    biom <- convertToBIOM(tse, assay.type = "relabundance")
+    # Test type
+    expect_equal(biomformat::matrix_element_type(biom), "float")
+    # Assay
+    test <- as.matrix( biomformat::biom_data(biom) )
+    ref <- assay(tse, "relabundance")
+    expect_equivalent(test, ref)
+    # rowData has one empty column (only NA values)
+    test <- as.data.frame( biomformat::observation_metadata(biom) )
+    expect_true( all(is.na(test)) && colnames(test) == "V1" )
+    # colData has one empty column (only NA values)
+    test <- as.data.frame( biomformat::sample_metadata(biom) )
+    expect_true( all(is.na(test)) && colnames(test) == "V1" )
+
 })
 
 test_that("Importing phyloseq objects yield TreeSummarizedExperiment objects", {
     skip_if_not_installed("phyloseq")
     data(GlobalPatterns, package="phyloseq")
-    me <- makeTreeSEFromPhyloseq(GlobalPatterns)
+    me <- convertFromPhyloseq(GlobalPatterns)
     expect_s4_class(me, "TreeSummarizedExperiment")
+    expect_is(assay(me), "matrix")
+    expect_true( !is(assay(me), "otu_table") )
+    expect_s4_class(colData(me), "DataFrame")
+    expect_s4_class(rowData(me), "DataFrame")
+    expect_is(rowTree(me), "phylo")
     expect_equal(dim(me),c(19216,26))
     data(enterotype, package="phyloseq")
-    me <- makeTreeSEFromPhyloseq(enterotype)
+    me <- convertFromPhyloseq(enterotype)
     expect_s4_class(me, "TreeSummarizedExperiment")
     expect_equal(dim(me),c(553,280))
     data(esophagus, package="phyloseq")
-    me <- makeTreeSEFromPhyloseq(esophagus)
+    me <- convertFromPhyloseq(esophagus)
     expect_s4_class(me, "TreeSummarizedExperiment")
     expect_equal(dim(me),c(58,3))
     esophagus2 <- esophagus
     phyloseq::otu_table(esophagus2) <- t(phyloseq::otu_table(esophagus))
-    me2 <- makeTreeSEFromPhyloseq(esophagus2)
+    me2 <- convertFromPhyloseq(esophagus2)
     expect_equal(me, me2)
 })
 
@@ -141,12 +186,12 @@ test_that("Importing dada2 objects yield TreeSummarizedExperiment objects", {
     dadaF <- dada2::dada(fnF, selfConsist=TRUE)
     dadaR <- dada2::dada(fnR, selfConsist=TRUE)
 
-    me <- makeTreeSEFromDADA2(dadaF, fnF, dadaR, fnR)
+    me <- convertFromDADA2(dadaF, fnF, dadaR, fnR)
     expect_s4_class(me, "TreeSummarizedExperiment")
 })
 
 test_that("Importing Mothur files yield SummarizedExperiment objects", {
-    
+
     counts <- system.file("extdata", "mothur_example.shared", package = "mia")
     taxa <- system.file("extdata", "mothur_example.cons.taxonomy", package = "mia")
     taxa2 <- system.file("extdata", "mothur_example.taxonomy", package = "mia")
@@ -159,17 +204,15 @@ test_that("Importing Mothur files yield SummarizedExperiment objects", {
     se2 <- importMothur(counts, taxa2)
     expect_s4_class(se, "SummarizedExperiment")
     expect_s4_class(se2, "SummarizedExperiment")
-    expect_error(importMothur(counts, meta))
-    expect_error(importMothur(counts, meta))
-    se <- importMothur(counts, designFile = meta)
-    se2 <- importMothur(counts, designFile = meta)
+    se <- importMothur(counts, col.file = meta)
+    se2 <- importMothur(counts, col.file = meta)
     expect_s4_class(se, "SummarizedExperiment")
     expect_s4_class(se2, "SummarizedExperiment")
-    se <- importMothur(counts, taxa, meta)
-    se2 <- importMothur(counts, taxa2, meta)
+    se <- importMothur(assay.file = counts, row.file = taxa, col.file = meta)
+    se2 <- importMothur(assay.file = counts, row.file = taxa2, col.file = meta)
     expect_s4_class(se, "SummarizedExperiment")
     expect_s4_class(se2, "SummarizedExperiment")
-    
+
     # Checks dimensions, rownames, and colnames of assay
     expect_equal(nrow(assays(se)$counts), 100)
     expect_equal(rownames(assays(se)$counts)[1:10],
@@ -177,7 +220,7 @@ test_that("Importing Mothur files yield SummarizedExperiment objects", {
                             "Otu006", "Otu007", "Otu008", "Otu009", "Otu010"))
     expect_equal(ncol(assays(se)$counts), 100)
     expect_equal(colnames(assays(se)$counts)[1:10],
-                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5", 
+                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5",
                             "Sample6", "Sample7", "Sample8", "Sample9", "Sample10"))
     expect_equal(nrow(assays(se2)$counts), 100)
     expect_equal(rownames(assays(se2)$counts)[1:10],
@@ -185,9 +228,9 @@ test_that("Importing Mothur files yield SummarizedExperiment objects", {
                     "Otu006", "Otu007", "Otu008", "Otu009", "Otu010"))
     expect_equal(ncol(assays(se2)$counts), 100)
     expect_equal(colnames(assays(se)$counts)[1:10],
-                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5", 
+                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5",
                             "Sample6", "Sample7", "Sample8", "Sample9", "Sample10"))
-    
+
     # Checks that rowData has right dimensions, rownames, and colnames
     expect_equal(nrow(rowData(se)), 100)
     expect_equal(rownames(rowData(se))[1:10],
@@ -201,32 +244,31 @@ test_that("Importing Mothur files yield SummarizedExperiment objects", {
                             "Otu006", "Otu007", "Otu008", "Otu009", "Otu010"))
     expect_equal(colnames(rowData(se2)),
                 c("OTU", "Kingdom", "Phylum", "Order", "Class", "Family", "Genus"))
-    
+
     expect_equal(nrow(colData(se)), 100)
     expect_equal(rownames(colData(se))[1:10],
-                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5", 
+                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5",
                             "Sample6", "Sample7", "Sample8", "Sample9", "Sample10"))
-    
+
     # Checks colData's dimensions and names of columns and rows
     expect_equal(colnames(colData(se)),
                         c("group", "sex", "age", "drug", "label", "numOtus", "Group"))
     expect_equal(nrow(colData(se2)), 100)
     expect_equal(rownames(colData(se2))[1:10],
-                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5", 
+                        c("Sample1", "Sample2", "Sample3", "Sample4", "Sample5",
                             "Sample6", "Sample7", "Sample8", "Sample9", "Sample10"))
     expect_equal(colnames(colData(se2)),
                         c("group", "sex", "age", "drug", "label", "numOtus", "Group"))
 })
 
-featureTableFile <- system.file("extdata", "table.qza", package = "mia")
-taxonomyTableFile <- system.file("extdata", "taxonomy.qza", package = "mia")
-sampleMetaFile <- system.file("extdata", "sample-metadata.tsv", package = "mia")
-refSeqFile <- system.file("extdata", "refseq.qza", package = "mia")
+assay.file <- system.file("extdata", "table.qza", package = "mia")
+row.file <- system.file("extdata", "taxonomy.qza", package = "mia")
+col.file <- system.file("extdata", "sample-metadata.tsv", package = "mia")
+refseq.file <- system.file("extdata", "refseq.qza", package = "mia")
 
 test_that("make TSE worked properly while no sample or taxa data", {
-    skip_if_not(require("biomformat", quietly = TRUE))
     ## no sample data or taxa data
-    expect_silent(tse <- importQIIME2(featureTableFile))
+    expect_silent(tse <- importQIIME2(assay.file))
     expect_s4_class(tse, "TreeSummarizedExperiment")
     expect_equal(dim(tse), c(770,34))
 })
@@ -235,21 +277,21 @@ test_that("reference sequences of TSE", {
     skip_if_not(require("biomformat", quietly = TRUE))
     # 1. fasta file of refseq
     tse <- importQIIME2(
-        featureTableFile,
-        refSeqFile = refSeqFile
+        assay.file,
+        refseq.file = refseq.file
     )
     tse2 <-  importQIIME2(
-        featureTableFile,
-        refSeqFile = refSeqFile,
+        assay.file,
+        refseq.file = refseq.file,
         featureNamesAsRefseq = FALSE
     )
-    expect_identical(tse@referenceSeq, importQZA(refSeqFile))
-    expect_identical(tse2@referenceSeq, importQZA(refSeqFile))
+    expect_identical(tse@referenceSeq, importQZA(refseq.file)[rownames(tse), ])
+    expect_identical(tse2@referenceSeq, importQZA(refseq.file)[rownames(tse), ])
 
     # 2. row.names of feature table as refseq
     # 2.1 element of row.names of feature table is not DNA sequence
     tse <- importQIIME2(
-        featureTableFile,
+        assay.file,
         featureNamesAsRefseq = TRUE
     )
     expect_null(tse@referenceSeq)
@@ -259,7 +301,7 @@ test_that("reference sequences of TSE", {
     # codes used for create sample data (donot run)
     if (FALSE) {
         .require_package("biomformat")
-        feature_tab <- importQZA(featureTableFile)
+        feature_tab <- importQZA(assay.file)
         n_feature <- nrow(feature_tab)
         random_seq <- sapply(
             rep(20, n_feature),
@@ -287,7 +329,7 @@ test_that("reference sequences of TSE", {
         package = "mia"
     )
 
-    # featureNamesAsRefseq is TRUE, refSeqFile is NULL, set row.names of
+    # featureNamesAsRefseq is TRUE, refseq.file is NULL, set row.names of
     # feature table as reference sequences
     tse <- importQIIME2(
         featureTableFile2,
@@ -295,22 +337,22 @@ test_that("reference sequences of TSE", {
     )
     feature_tab <- importQZA(featureTableFile2)
     names_seq <- Biostrings::DNAStringSet(row.names(feature_tab))
-    names(names_seq) <- paste0("seq_", seq_along(names_seq))
+    names(names_seq) <- names_seq
     expect_identical(tse@referenceSeq, names_seq)
 
-    # refSeqFile is not NULL, featureNamesAsRefseq is TRUE,
-    # set the sequences from refSeqFile as reference sequences
+    # refseq.file is not NULL, featureNamesAsRefseq is TRUE,
+    # set the sequences from refseq.file as reference sequences.
     tse <- importQIIME2(
-        featureTableFile2,
+        assay.file,
         featureNamesAsRefseq = TRUE,
-        refSeqFile = refSeqFile
+        refseq.file = refseq.file
     )
-    expect_identical(tse@referenceSeq, importQZA(refSeqFile))
+    expect_identical(tse@referenceSeq, importQZA(refseq.file)[rownames(tse)])
 
-    # 3. refSeqFile = NULL, featureNamesAsRefseq = FALSE
+    # 3. refseq.file = NULL, featureNamesAsRefseq = FALSE
     tse <- importQIIME2(
-        featureTableFile,
-        refSeqFile = NULL,
+        assay.file,
+        refseq.file = NULL,
         featureNamesAsRefseq = FALSE
     )
     expect_null(tse@referenceSeq)
@@ -336,7 +378,7 @@ test_that("`.parse_taxonomy` work with any combination of taxonomic ranks", {
         byrow = TRUE,
         dimnames = list(c("a", "b"), c("Feature.ID", "Taxon", "Confidence"))
     )
-    expect_true(is.na(mia:::.parse_taxonomy(test_taxa)[2,"Species"]))
+    expect_true(is.na(mia:::.parse_taxonomy(test_taxa)[2,"species"]))
 
     # if the expexted order is not present it will return a correct result
     test_taxa <- matrix(
@@ -346,13 +388,15 @@ test_that("`.parse_taxonomy` work with any combination of taxonomic ranks", {
         byrow = TRUE,
         dimnames = list(c("a", "b"), c("Feature.ID", "Taxon", "Confidence"))
     )
-    expect_equal(mia:::.parse_taxonomy(test_taxa)[,"Species"],c("s__test",NA))
-    expect_equal(mia:::.parse_taxonomy(test_taxa, removeTaxaPrefixes = TRUE)[,"Species"],
+    expect_equal(mia:::.parse_taxonomy(test_taxa)[,"species"],c("s__test",NA))
+    expect_equal(mia:::.parse_taxonomy(test_taxa, prefix.rm = TRUE)[,"species"],
                 c("test",NA))
+    # Expect that output have only taxonomy levels that have information
+    expect_equal(ncol(mia:::.parse_taxonomy(test_taxa)), 3L)
 })
 
 test_that("`.read_q2sample_meta` remove  the row contained `#q2:types`", {
-    expect_false(any(as(.read_q2sample_meta(sampleMetaFile), "matrix") == "#q2:types"))
+    expect_false(any(as(.read_q2sample_meta(col.file), "matrix") == "#q2:types"))
 })
 
 test_that('get file extension', {
@@ -362,40 +406,42 @@ test_that('get file extension', {
 
 test_that('read qza file', {
     expect_error(importQZA("abc"), "does not exist")
-    expect_error(importQZA(sampleMetaFile), "must be in `qza` format")
+    expect_error(importQZA(col.file), "must be in `qza` format")
 })
 
 test_that("Confidence of taxa is numberic", {
     skip_if_not(require("biomformat", quietly = TRUE))
     tse <- importQIIME2(
-        featureTableFile,
-        taxonomyTableFile = taxonomyTableFile
+        assay.file,
+        row.file = row.file
     )
     expect_true(is.numeric(S4Vectors::mcols(tse)$Confidence))
 })
 
 test_that("dimnames of feature table is identicle with meta data", {
     skip_if_not(require("biomformat", quietly = TRUE))
-    feature_tab <- importQZA(featureTableFile)
-   
-    sample_meta <- .read_q2sample_meta(sampleMetaFile)
-    taxa_meta <- importQZA(taxonomyTableFile)
-    taxa_meta <- .subset_taxa_in_feature(taxa_meta, feature_tab)
-    new_feature_tab <- .set_feature_tab_dimnames(
-        feature_tab, 
-        sample_meta, 
+    feature_tab <- importQZA(assay.file)
+
+    sample_meta <- .read_q2sample_meta(col.file)
+    taxa_meta <- importQZA(row.file)
+    data_list <- .set_feature_tab_dimnames(
+        feature_tab,
+        sample_meta,
         taxa_meta
    )
-    expect_identical(rownames(new_feature_tab), rownames(taxa_meta))
-    expect_identical(colnames(new_feature_tab), rownames(sample_meta))
-   
+    expect_identical(colnames(data_list[[1]]), colnames(feature_tab))
+    expect_identical(rownames(data_list[[1]]), rownames(feature_tab))
+    expect_identical(rownames(data_list[[1]]), rownames(data_list[[2]]))
+    expect_identical(colnames(data_list[[1]]), rownames(data_list[[3]]))
+    expect_identical(colnames(data_list[[1]]), colnames(data_list[[1]]))
+
     # sample_meta or feature meta is NULL
     sample_meta2 <- S4Vectors::make_zero_col_DFrame(ncol(feature_tab))
     rownames(sample_meta2) <- colnames(feature_tab)
     taxa_meta2 <- S4Vectors::make_zero_col_DFrame(nrow(feature_tab))
     rownames(taxa_meta2) <- rownames(feature_tab)
     expect_silent(.set_feature_tab_dimnames(feature_tab, sample_meta2, taxa_meta))
-   
+
     # sample meta or feature meta without any information, only contains sample/feature
     # ID in its rownames
     feature_tab3 <- S4Vectors::DataFrame(
@@ -406,17 +452,19 @@ test_that("dimnames of feature table is identicle with meta data", {
     )
     sample_meta3 <- S4Vectors::DataFrame(row.names = paste0("sample", 3:1))
     feature_meta3 <- S4Vectors::DataFrame(row.names = paste0("feature", c(2, 3, 1)))
-    new_feature_tab3 <- .set_feature_tab_dimnames(
-        feature_tab3, 
-        sample_meta3, 
+    data_list <- .set_feature_tab_dimnames(
+        feature_tab3,
+        sample_meta3,
         feature_meta3
     )
-    expect_identical(row.names(new_feature_tab3), paste0("feature", c(2, 3, 1)))
-    expect_identical(colnames(new_feature_tab3), paste0("sample", 3:1))
+    expect_identical(rownames(data_list[[1]]), rownames(feature_tab3))
+    expect_identical(colnames(data_list[[1]]), colnames(feature_tab3))
+    expect_identical(rownames(data_list[[2]]), rownames(feature_tab3))
+    expect_identical(rownames(data_list[[3]]), colnames(feature_tab3))
 })
 
 
-test_that("makePhyloseqFromTreeSE", {
+test_that("convertToPhyloseq", {
 
     skip_if_not_installed("phyloseq")
 
@@ -424,7 +472,7 @@ test_that("makePhyloseqFromTreeSE", {
     data(GlobalPatterns, package="mia")
     tse <- GlobalPatterns
 
-    phy <- makePhyloseqFromTreeSE(GlobalPatterns)
+    phy <- convertToPhyloseq(GlobalPatterns)
 
     # Test that assay is in otu_table
     expect_equal(as.data.frame(phyloseq::otu_table(phy)@.Data), as.data.frame(assays(tse)$counts))
@@ -442,51 +490,52 @@ test_that("makePhyloseqFromTreeSE", {
     # Test that referenceSeq is in refseq. Expect error, because there should not be
     # reference sequences.
     expect_error(phyloseq::refseq(phy))
-    
+
     # Test with agglomeration that that pruning is done internally
-    test1 <- agglomerateByRank(tse, rank = "Phylum")
-    test2 <- agglomerateByRank(tse, rank = "Phylum", agglomerate.tree = TRUE)
-    test1_phy <- expect_warning(makePhyloseqFromTreeSE(test1))
-    test2_phy <- makePhyloseqFromTreeSE(test2)
-    
-    expect_equal(length(phyloseq::phy_tree(test1_phy)$node), 
+    test1 <- agglomerateByRank(tse, rank = "Phylum", update.tree = FALSE)
+    test2 <- agglomerateByRank(tse, rank = "Phylum", update.tree = TRUE)
+    test1_phy <- expect_warning(convertToPhyloseq(test1))
+    test2_phy <- convertToPhyloseq(test2)
+
+    expect_equal(length(phyloseq::phy_tree(test1_phy)$node),
                 length(ape::keep.tip(rowTree(test1), rowLinks(test1)$nodeLab)$node))
     expect_equal(phyloseq::phy_tree(test1_phy)$tip.label, rownames(test2))
     # The tip labels do not match because of renaming
     expect_identical(phyloseq::phy_tree(test2_phy)$edge, rowTree(test2)$edge)
-    
+
     # Check that everything works also with agglomerated data
     for (level in colnames(rowData(tse)) ){
         temp <- agglomerateByRank(tse, rank = level)
-        expect_warning(makePhyloseqFromTreeSE(temp))
+        expect_no_warning(convertToPhyloseq(temp))
     }
-    
+
     tse2 <- tse
     # Concerts data frame to factors
     rowData(tse2) <- DataFrame(lapply(rowData(tse2), as.factor))
-    phy <- makePhyloseqFromTreeSE(tse)
-    phy2 <- makePhyloseqFromTreeSE(tse2)
+    phy <- convertToPhyloseq(tse)
+    phy2 <- convertToPhyloseq(tse2)
     expect_equal(phyloseq::tax_table(phy2), phyloseq::tax_table(phy))
-    
+
     # TSE object
     data(esophagus, package="mia")
     tse <- esophagus
 
-    phy <- makePhyloseqFromTreeSE(tse, assay.type="counts")
+    phy <- convertToPhyloseq(tse, assay.type="counts")
 
     # Test that assay is in otu_table
     expect_equal(as.data.frame(phyloseq::otu_table(phy)@.Data), as.data.frame(assays(tse)$counts))
 
     # Test that rowTree is in phy_tree
     expect_identical(phyloseq::phy_tree(phy), rowTree(tse))
-    
+
     # Test that merging objects lead to correct phyloseq
-    tse <- mergeSEs(GlobalPatterns, esophagus, assay.type="counts", missing_values = 0)
-    pseq <- makePhyloseqFromTreeSE(tse, assay.type="counts")
-    
-    tse_compare <- tse[ rownames(GlobalPatterns), ]
-    pseq_compare <- makePhyloseqFromTreeSE(tse_compare, assay.type="counts")
-    
+    tse <- mergeSEs(GlobalPatterns, esophagus, assay.type="counts", missing.values = 0)
+    pseq <- convertToPhyloseq(tse, assay.type="counts")
+
+    # Include rownames from both trees
+    tse_compare <- tse[ c(rownames(GlobalPatterns), rownames(esophagus)), ]
+    pseq_compare <- convertToPhyloseq(tse_compare, assay.type="counts")
+
     expect_equal(phyloseq::otu_table(pseq), phyloseq::otu_table(pseq_compare))
 })
 
@@ -494,8 +543,8 @@ test_that("Import HUMAnN file", {
     file_path <- system.file("extdata", "humann_output.tsv", package = "mia")
     tse <- importHUMAnN(file_path)
     #
-    expect_true( length(taxonomyRanks(tse)) == 7 )
-    expect_true( ncol(rowData(tse)) == 9 )
+    expect_true( length(taxonomyRanks(tse)) == 2 )
+    expect_true( ncol(rowData(tse)) == 4 )
     expect_true( ncol(tse) == 3 )
     expect_true( nrow(tse) == 12 )
     expect_true( all(!is.na(assay(tse))) )
@@ -514,3 +563,27 @@ test_that("Import HUMAnN file", {
     # colData should not change the result
     expect_equal(tse, tse2)
 })
+
+test_that("Import MetaPhlAn file", {
+    file_path <- system.file("extdata", "merged_abundance_table.txt", package = "mia")
+    tse <- importMetaPhlAn(file_path)
+    # Check dimensions
+    expect_true( length(taxonomyRanks(tse)) == 7L )
+    expect_true( length(altExps(tse)) == 7L )
+    # All experiments should have same number of samples
+    expect_true( sapply(altExps(tse), function(x) ncol(x) == 6L) |> all() )
+    # The last experiment in altExp should equal to the "lowest level" as there
+    # are no undetected features.
+    expect_equal( dim(altExp(tse, length(altExps(tse)))), dim(tse))
+
+    # Check that the function works with features that are not detected
+    df <- read.table(file_path, header = TRUE, sep = "\t", check.names = FALSE)
+    num_missing <- 4L
+    random_species <- sample(which(grepl("s__", df[["clade_name"]])), num_missing)
+    df[random_species, "clade_name"] <- "not_detected"
+    # Save to a temporary file that will be next read by the function
+    temp_path <- tempfile(fileext = ".txt")
+    write.table(df, file = temp_path, sep = "\t", quote = FALSE, row.names = FALSE)
+    tse <- importMetaPhlAn(temp_path)
+    expect_true( nrow(altExp(tse, length(altExps(tse))))+num_missing == nrow(tse))
+})
\ No newline at end of file
