@@ -38,17 +38,16 @@ test_that("Unifrac beta diversity", {
     # Calculate unweighted unifrac
     unifrac_mia <- as.matrix(getDissimilarity(tse, method = "unifrac",
                                               weighted = FALSE))
-    unifrac_rbiom <- as.matrix(rbiom::unifrac(assay(tse), weighted = FALSE,
+    unifrac_ecodive <- as.matrix(ecodive::unweighted_unifrac(t(assay(tse)),
                                               rowTree(tse)))
-    expect_equal(unifrac_mia, unifrac_rbiom)
+    expect_equal(unifrac_mia, unifrac_ecodive)
     # Calculate weighted unifrac. Allow tolerance since weighted unifrac
-    # calculation in rbiom has some stochasticity. That is most likely due
-    # multithreading and complex structure of tree (loops).
+    # calculation may have some stochasticity.
     unifrac_mia <- as.matrix(getDissimilarity(tse, method = "unifrac",
                                               weighted = TRUE))
-    unifrac_rbiom <- as.matrix(rbiom::unifrac(assay(tse), weighted = TRUE,
+    unifrac_ecodive <- as.matrix(ecodive::weighted_unifrac(t(assay(tse)),
                                               rowTree(tse)))
-    expect_equal(unifrac_mia, unifrac_rbiom, tolerance = 5e-3)
+    expect_equal(unifrac_mia, unifrac_ecodive, tolerance = 5e-3)
 
     # Test that the function works correctly when there are multiple trees.
     # The function should subset the data based on tree.
@@ -66,18 +65,18 @@ test_that("Unifrac beta diversity", {
                                     weighted = FALSE, tree.name = "tree2")
     )
     unifrac_mia <- as.matrix(unifrac_mia)
-    unifrac_rbiom <- as.matrix(rbiom::unifrac(assay(tse_ref), weighted = FALSE,
+    unifrac_ecodive <- as.matrix(ecodive::unweighted_unifrac(t(assay(tse_ref)),
                                               rowTree(tse_ref)))
-    expect_equal(unifrac_mia, unifrac_rbiom)
+    expect_equal(unifrac_mia, unifrac_ecodive)
     # Calculate weighted unifrac
     expect_warning(
     unifrac_mia <- getDissimilarity(tse, method = "unifrac",
                                     weighted = TRUE, tree.name = "tree2")
     )
     unifrac_mia <- as.matrix(unifrac_mia)
-    unifrac_rbiom <- as.matrix(rbiom::unifrac(assay(tse_ref), weighted = TRUE,
+    unifrac_ecodive <- as.matrix(ecodive::weighted_unifrac(t(assay(tse_ref)),
                                               rowTree(tse_ref)))
-    expect_equal(unifrac_mia, unifrac_rbiom, tolerance = 1e-3)
+    expect_equal(unifrac_mia, unifrac_ecodive, tolerance = 1e-3)
 
     # Test the function with agglomerated data. .get_unifrac renames
     # rownames based on tips and links to them. Then it also prunes the tree so
@@ -90,13 +89,13 @@ test_that("Unifrac beta diversity", {
     # Calculate unweighted unifrac
     unifrac_mia <- as.matrix(getDissimilarity(tse, method = "unifrac",
                                               weighted = FALSE))
-    unifrac_rbiom <- as.matrix(rbiom::unifrac(assay(tse_ref), weighted = FALSE,
+    unifrac_ecodive <- as.matrix(ecodive::unweighted_unifrac(t(assay(tse_ref)),
                                               rowTree(tse_ref)))
     # Calculate weighted unifrac. No tolerance needed since the tree has
     # simpler structure after pruning.
     unifrac_mia <- as.matrix(getDissimilarity(tse, method = "unifrac",
                                               weighted = TRUE))
-    unifrac_rbiom <- as.matrix(rbiom::unifrac(assay(tse_ref), weighted = TRUE,
+    unifrac_ecodive <- as.matrix(ecodive::weighted_unifrac(t(assay(tse_ref)),
                                               rowTree(tse_ref)))
-    expect_equal(unifrac_mia, unifrac_rbiom)
+    expect_equal(unifrac_mia, unifrac_ecodive)
 })
